/// <reference path="../typings/phaser.d.ts" />
/// <reference path="../typings/globals/jquery/index.d.ts" />

const gameRatio = window.innerWidth / window.innerHeight;
const w = window.innerWidth * window.devicePixelRatio;
const h = window.innerHeight * window.devicePixelRatio;


/**
 * App
 */
module LosPlantas {
  export const PLANT_MAX_VALUE = 100;
  export const PLANT_MIN_VALUE = 0;
  export const PLANT_START_VALUE = 50;
  
  export const MINIGAME_BATH = '0';
  export const MINIGAME_KITCHEN = '1';
  export const MINIGAME_BABYSITTER = '2';
  export const MINIGAME_TV = '3';

  export const SIZE_ROOMS = {
    width: 400,
    height: 200
  }

  export const POS_LIVING_ROOM = {
    x: 102 + 200,
    y: 385 + 100
  }
  export const POS_BATH_ROOM = {
    x: 522 + 200,
    y: 165 + 100
  }
  export const POS_PLANTS = {
    x: 522 + 200 + 40,
    y: 165 + 100 - 120 - 40 - 36
  }
  export const POS_KITCHEN = {
    x: 522 + 200,
    y: 385 + 100
  }
  export const POS_BED_ROOM = {
    x: 102 + 200,
    y: 165 + 100
  }
  export const POS_ATTIC = {
    x: 102 + 262,
    y: 165 + 100 - 120 - 40 - 36
  }

  export let NPC_FATHER: NPC;
  export let NPC_MOTHER: NPC;
  export let NPC_BROTHER: NPC;
  export let NPC_GIRL: NPC;
  export let STATUS_MATCH: string;

  export let player: Player;

  export let timerFont = { 
    font: "60px Ruda,Arial", fill: "#5073d5",
    align: "left",
    fontWeight: "bold",
    boundsAlignH: "left",
    boundsAlignV: "middle",
    wordWrap: false };

  export class App extends Phaser.Game {
    game: Phaser.Game;

    constructor() {
      NPC_FATHER = new NPC('father_sprite');
      NPC_FATHER.addAnimation('crazy', [0, 1, 2, 3, 4], 8, true);
      NPC_MOTHER = new NPC('mother_sprite');
      NPC_MOTHER.addAnimation('crazy', [0, 1, 2, 3, 4], 8, true);
      NPC_BROTHER = new NPC('brother_sprite');
      NPC_BROTHER.addAnimation('crazy', [0, 1, 2, 3, 4], 8, true);
      NPC_GIRL = new NPC('girl_idle');
      NPC_GIRL.addAnimation('idle', [0, 1, 2, 3], 1, true);
      

      super(1024, 600, Phaser.AUTO, 'content', null);
      this.state.add('First', FirstState, false);
      this.state.add('TheEnd', EndState, false);
      this.state.add('Loader', Loader, false);
      this.state.add('Menu', Menu, false);
      this.state.add('Intro', Intro, false);
      this.state.add('MinigameSelection', MinigameSelection, false);
      this.state.add('MinigameTransition', MinigameTransition, false);
      this.state.add('Minigametv', Minigametv, false);
      this.state.add('Minigamebath', Minigamebath, false);
      this.state.add('Minigamekitchen', Minigamekitchen, false);

      this.state.add('PruebaMinigame', PruebaMinigame, false);
      
      player = new Player();

      this.state.start('First');
    }
  }
}

window.onload = () => {
  $.get('/resources.json', (data: string) => {
    (window as any).resources = data;
    const app = new LosPlantas.App();
  });
}
