
module LosPlantas { 
  export function loadResources(game: Phaser.Game){
    (window as any).resources.images.forEach((image: Array<string>) => {
      game.load.image(image[0], image[1]);
    });
    (window as any).resources.text.forEach((text: Array<string>) => {
      game.load.text(text[0], text[1]);
    });
    (window as any).resources.json.forEach((json: Array<string>) => {
      game.load.json(json[0], json[1]);
    });
    (window as any).resources.videos.forEach((video: Array<string>) => {
      game.load.video(video[0], video[1]);
    });

    interface Spritesheet {
      name: string,
      url: string,
      height: number,
      width: number,
      qt: number
    }

    (window as any).resources.spritesheets.forEach((spritesheet: Spritesheet) => {
      game.load.spritesheet(
        spritesheet.name,
        spritesheet.url,
        spritesheet.width,
        spritesheet.height,
        spritesheet.qt
      );
    });
    
  }


  export function showVibes(game: Phaser.Game, isGood: boolean = false, reversed: boolean = false, qt: number = 1, duration: number = 1, x?: number, y?: number, opacity: number = 1){
    
    for (let i=0; i < qt; i++) {
      
      if (duration > 0) {
        // console.log('Duration', duration);

        const inter = setInterval(() => {
          showVibe(game, x, y, isGood, reversed, opacity);
        }, 200);

        setTimeout(() => clearInterval(inter), duration * 1000);
        // const timer = game.time.events.loop( 400 , () => {
          
        // });
        // timer.timer.start();
        //setTimeout(() => timer.timer.stop(), duration * 1000);
      }else{
        showVibe(game, x, y, isGood, reversed, opacity);
      }
      
    }
  }
}

function showVibe(game: Phaser.Game, x?: number, y?: number, isGood?: boolean, reversed: boolean = false, opacity: number = 1){
  let posX = 0;
  let posY = 0;
  let vibe = null;

  if (!!x && !!y){
    posX = x;
    posY = y;  
  }else{
    posX = (Math.random() * 1024);
    posY = (Math.random() * 600);
  }
  vibe = game.add.sprite(posX, posY, ((isGood) ? 'good' : 'bad') + '_vibe');
  vibe.anchor.set(0.5, 0.5);

  if (!reversed){
    vibe.alpha = opacity;
    vibe.height = 0;
    vibe.width = 0;
  }else{
    vibe.alpha = 0;
    vibe.height = 300;
    vibe.width = 300;
  }
  
  let postStatus = { alpha: 0, width: 300, height: 300 };

  if (reversed){
    postStatus = { alpha: opacity, width: 0, height: 0 }
  }

  game.add.tween(vibe).to(postStatus, 2000, Phaser.Easing.Linear.None, true, 0, 0);
}
