
interface AnimationData {
  name: string,
  indexs: Array<number>,
  fps: number,
  loop: boolean
}

module LosPlantas {

  export class NPC {
    sprite: Phaser.Sprite;
    key: string;
    animations: Array<AnimationData>;

    constructor(key: string) {
      this.key = key;
      this.sprite = null;
      this.animations = new Array<AnimationData>();
    }

    addAnimation(name: string, indexs: Array<number>, fps: number, loop: boolean = false){
      this.animations.push({
        name,
        indexs,
        fps,
        loop
      });
    }

    playAnimation(name: string){
      // if (!this.sprite)
      //   return console.log('No has creado antes un sprite');

      this.sprite.play(name);
    }

    execHorizontalTween(game: Phaser.Game, x: number, time: number = 2000, f: Function = Phaser.Easing.Linear.None){
      return game.add.tween(this.sprite).to({
        x: x
      }, time, f, true, 0, 9999, true);
    }

    insertInWorld(game: Phaser.Game, x: number, y: number, width?: number, height?: number, frame: number = 0){

      this.sprite = game.add.sprite(x, y, this.key, frame);
      this.sprite.anchor.set(0.5, 0.5);

      if (!!width && !!height){
        this.sprite.width = width;
        this.sprite.height = height;
      }
        
      this.animations.forEach(animationData => {
        this.sprite.animations.add(
          animationData.name,
          animationData.indexs,
          animationData.fps,
          animationData.loop
        );
      });
    }

  }

}
