module LosPlantas {
    
  const TRANSITION_STATE_START = 0;
  const TRANSITION_STATE_WAIT = 100;
  const TRANSITION_STATE_END = 200;

  const TRANSITION_FSM_RATE = 4; // number of times the FSM is executed per second;

  export class MinigameTransition extends Phaser.State {
      
    private girlRunSprite : Phaser.Sprite;
    private localState : number;
    private localCounter : number;
    private localDuration : number;
    private localTimer : Phaser.TimerEvent;
    private girlTween : Phaser.Tween;

    create() {
      this.game.add.image(0, 0, 'transition_bg');
      // console.log(player.nextMiniGame + " " + Date.now().toString());
      this.girlRunSprite = this.add.sprite(this.world.centerX+700, 200, 'girl_running');
      this.girlRunSprite.anchor.set(0.5,0.5);
      this.girlRunSprite.animations.add('walk',[0,1],6,true);
      this.girlRunSprite.play('walk');
      
      this.localState = TRANSITION_STATE_START;
      this.localTimer = this.game.time.events.loop(Phaser.Timer.SECOND * (1 / TRANSITION_FSM_RATE), this.doGameLogic, this);
      this.localTimer.timer.start();
    }

    doGameLogic() {
      switch(this.localState) {
        case TRANSITION_STATE_START:
          this.girlTween = this.game.add.tween(this.girlRunSprite).to( 
              { x: this.world.centerX - 100, y: 440 }, 750, Phaser.Easing.Linear.None,true);
          this.girlTween.onComplete.add(() => {
            this.game.add.tween(this.girlRunSprite).to( 
              { x: this.world.centerX - 700}, 750, Phaser.Easing.Linear.None,true);
          });
          this.localCounter = 0;
          this.localDuration = 1.5 * TRANSITION_FSM_RATE;
          this.localState = TRANSITION_STATE_WAIT;
          break;
        case TRANSITION_STATE_WAIT:
          if (this.localCounter < this.localDuration) {
              this.localCounter++;
          } else {
              this.localState = TRANSITION_STATE_END;
          }
          break;
        case TRANSITION_STATE_END:
          let phase: string;
          switch(player.nextMiniGame) {
            case MINIGAME_BATH:
              phase = 'Minigamebath';
              break;
            case MINIGAME_KITCHEN:
              phase = 'Minigamekitchen';
              break;
            case MINIGAME_TV:
              phase = 'Minigametv';
              break;
            case MINIGAME_BABYSITTER:
              phase = 'PruebaMinigame';
              break;
            default:
              phase = 'PruebaMinigame';
              break;
          }
          this.game.state.start(phase,true,false);
          break;
      }
    } // end of doGameLogic
  } // end of MinigameTransition
} // end of module LosPlantas
