module LosPlantas {

  const MINIGAMEBATH_STATE_START = 0;
  const MINIGAMEBATH_STATE_END = 1000;
  const MINIGAMEBATH_STATE_SUCCESS_START = 10;
  const MINIGAMEBATH_STATE_SUCCESS_WAIT = 11;
  const MINIGAMEBATH_STATE_FAILURE_START = 12;
  const MINIGAMEBATH_STATE_FAILURE_WAIT = 13;


  const MINIGAMEBATH_FSM_RATE = 16; // how many times per second to execute FSM
  const MINIGAMEBATH_TIMER_MAX = 8; // seconds
  const MINIGAMEBATH_WARNING_COLOR1 = '#FF0000';
  const MINIGAMEBATH_WARNING_COLOR2 = '#FFFF00';
  const MINIGAMEBATH_MAX_VIBES = 20;
  const MINIGAMEBATH_MAX_DUCKS = 20;

  export class Minigamebath extends Minigame {

    private localTimer: Phaser.TimerEvent;
    private localState: number;
    private localCounter: number;
    private localDuration: number;
    private localSeconds: number;
    private localSecondsChangeColor: boolean;
    private localDucksShooted: number;

    private backgroundImage: Phaser.Image;
    private marqueeSprite: Phaser.Sprite;
    private wave1Sprite: Phaser.Sprite;
    private wave1SpriteTween: Phaser.Tween;
    private wave2Sprite: Phaser.Sprite;
    private wave2SpriteTween: Phaser.Tween;
    private boySprite: Phaser.Sprite;
    private duckSprites: Array<Phaser.Sprite>;
    private duckSpriteTweens: Array<Phaser.Tween>;
    private knifeSprite: Phaser.Sprite;
    private vibes: Array<Phaser.Sprite>;
    

    constructor(level: number){
      super(MINIGAME_BATH, level);
    }

    setTargetScore(){
      // this minigame is about choosing one correct image from an image cycle, so 1 point is enough  
      this.targetScore = 1;
    }

    create() { 
      this.backgroundImage = this.add.image(this.world.centerX, this.world.centerY, 'minigamebath_bg');
      this.backgroundImage.anchor.setTo(0.5,0.5);

      this.wave1Sprite = this.add.sprite(this.world.centerX+50, this.world.centerY+250, 'wave1');
      this.wave1Sprite.anchor.set(0.5, 0.5);
      
      NPC_BROTHER.insertInWorld(this.game, this.world.centerX - 400, this.world.centerY+50, 250, 250);
      NPC_BROTHER.playAnimation('crazy');

      this.wave1SpriteTween = this.game.add.tween(this.wave1Sprite).to( 
              { x: this.world.centerX, y: this.world.centerY+230 }, 2000, Phaser.Easing.Circular.InOut,true,0,-1,true);

      this.wave2Sprite = this.add.sprite(this.world.centerX-50, this.world.centerY+265, 'wave2');
      this.wave2Sprite.anchor.set(0.5, 0.5);
      this.wave2SpriteTween = this.game.add.tween(this.wave2Sprite).to( 
              { x: this.world.centerX, y: this.world.centerY+245 }, 2000, Phaser.Easing.Circular.InOut,true,0,-1,true);
      
      let offsetx = 0;
      let offsety = 0;
      let offsetdelay = 0;
      let offsetspeed = 0;
      this.duckSprites = new Array<Phaser.Sprite>();
      this.duckSpriteTweens = new Array<Phaser.Tween>();
      for (let i=0; i<MINIGAMEBATH_MAX_DUCKS; i++) {
        offsetx = Math.round(Math.random()*1024);
        offsety = 100+Math.round(Math.random()*300);
        offsetdelay = Math.round(Math.random()*15000);
        offsetspeed = 3000 + Math.round(Math.random()*offsetx)*3;
        this.duckSprites.push(this.add.sprite(this.world.centerX+600+offsetx, this.world.centerY+offsety, 'psychoduck'));
        this.duckSprites[i].anchor.setTo(0.5,0.5);
        this.duckSprites[i].animations.add("swim",[0,1,2,1],8,true);
        this.duckSprites[i].animations.add("boom",[3,4,5,6,7],8,false);
        this.duckSprites[i].play('swim');
        this.duckSpriteTweens.push(this.game.add.tween(this.duckSprites[i]).to(
        { x: this.world.centerX-425, y: this.world.centerY+50 }, offsetspeed, Phaser.Easing.Linear.None,true,offsetdelay,0,false));
        this.duckSpriteTweens[i].onComplete.add(this.duckReachesBoy, this);
      }

      //this.knifeSprite = this.add.sprite(this.world.centerX,this.world.centerY+200,'knife');
      //this.knifeSprite.anchor.setTo(0.5,0.5);
      
      this.vibes = new Array<Phaser.Sprite>();
      for (let i=0; i<MINIGAMEBATH_MAX_VIBES; i++) {
        this.vibes.push(this.add.sprite(this.world.centerX + (Math.random()*1000-500), 
          this.world.centerY + (Math.random()*600-300),
          'bad_vibe'));
        this.vibes[i].anchor.setTo(0.5,0.5);
        this.vibes[i].alpha = 1;
        this.vibes[i].height = 0;
        this.vibes[i].width = 0;
      }

      this.input.mouse.capture = true;
      //this.input.mouse.onMouseMove

      this.localDucksShooted = 0;
      this.localCounter = 0;
      this.localState = MINIGAMEBATH_STATE_START;
      this.localTimer = this.game.time.events.loop(Phaser.Timer.SECOND * (1 / MINIGAMEBATH_FSM_RATE), this.doGameLogic, this);
      this.localTimer.timer.start();   
    }

    duckReachesBoy(spr:Phaser.Sprite, twn:Phaser.Tween) {
      if (this.localState == MINIGAMEBATH_STATE_START) {
        let stopDucks = false;
        if (spr.frame < 3) {
          // the duck is intact, game over!!
          this.localState = MINIGAMEBATH_STATE_FAILURE_START;
          stopDucks = true;
        } else {
          // the duck is dead, check if the player shooted all the ducks
          this.localDucksShooted++;
          if (this.localDucksShooted == MINIGAMEBATH_MAX_DUCKS) {
            this.localState = MINIGAMEBATH_STATE_SUCCESS_START;
            stopDucks = true;
          }
        }
        if (stopDucks) {
          for(let i=0; i<MINIGAMEBATH_MAX_DUCKS; i++) {
            if (this.duckSpriteTweens[i].isRunning) {
              this.duckSpriteTweens[i].pause();
            }
          }
        }
      }
    }

    doGameLogic() {
      switch(this.localState) {
        case MINIGAMEBATH_STATE_START:
          if (this.input.activePointer.leftButton.isDown) {
            // theck if mouse position is near an active duck
            for (let i=0; i<MINIGAMEBATH_MAX_DUCKS; i++) {
              if (this.duckSprites[i].frame < 3) {
                let deltax = Math.abs(this.input.activePointer.worldX - this.duckSprites[i].x);
                let deltay = Math.abs(this.input.activePointer.worldY - this.duckSprites[i].y);
                //console.log("pointer: " + this.input.activePointer.worldX.toString() + 
                //  " - duck" + this.duckSprites[i].x.toString() + " - " + deltax.toString() + " - " + deltay.toString());
                if ((deltax < 40) && (deltay < 40)) {
                  // shoot that duck!
                  this.duckSprites[i].play('boom');
                }
              }
            }
          }          
          break;
        case MINIGAMEBATH_STATE_END:
          this.endMiniGame();
          break;
        case MINIGAMEBATH_STATE_SUCCESS_START:
          for (let i=0; i<MINIGAMEBATH_MAX_VIBES; i++) {
            this.vibes[i].loadTexture('good_vibe');
            this.game.add.tween(this.vibes[i]).to(
              { alpha: 0, width: 300, height: 300 },2000, Phaser.Easing.Linear.None,true,0,0);
          }
          this.actualScore = this.targetScore;
          this.localCounter = 0;
          this.localDuration = 2 * MINIGAMEBATH_FSM_RATE;
          this.localState = MINIGAMEBATH_STATE_SUCCESS_WAIT;
          break;
        case MINIGAMEBATH_STATE_SUCCESS_WAIT:
          if (this.localCounter > this.localDuration) {
            this.localState = MINIGAMEBATH_STATE_END;
          } else {
            this.localCounter++;
          }
          break;
        case MINIGAMEBATH_STATE_FAILURE_START:
          for (let i=0; i<MINIGAMEBATH_MAX_VIBES; i++) {
            this.game.add.tween(this.vibes[i]).to(
              { alpha: 0, width: 300, height: 300 },2000, Phaser.Easing.Linear.None,true,0,0);
          }
          this.localCounter = 0;
          this.localDuration = 2 * MINIGAMEBATH_FSM_RATE;
          this.localState = MINIGAMEBATH_STATE_FAILURE_WAIT;
          break;
        case MINIGAMEBATH_STATE_FAILURE_WAIT:
          if (this.localCounter > this.localDuration) {
            this.localState = MINIGAMEBATH_STATE_END;
          } else {
            this.localCounter++;
          }
          break;
      }
    }
  }

}
