module LosPlantas {

  const MINIGAMETV_STATE_START = 0;
  const MINIGAMETV_STATE_END = 1000;
  const MINIGAMETV_STATE_INSTRUCTIONS = 1;
  const MINIGAMETV_STATE_SHOW_REMOTE_START = 2;
  const MINIGAMETV_STATE_SHOW_REMOTE_WAIT = 3;
  const MINIGAMETV_STATE_SHOW_REMOTE_FINISH = 4;
  const MINIGAMETV_STATE_WAIT_USER_SELECTION = 5;
  const MINIGAMETV_STATE_PROCESS_SELECTION = 6;
  const MINIGAMETV_STATE_SUCCESS_START = 7;
  const MINIGAMETV_STATE_SUCCESS_WAIT = 8;
  const MINIGAMETV_STATE_FAILURE_START = 9;
  const MINIGAMETV_STATE_FAILURE_WAIT = 10;

  const MINIGAMETV_FSM_RATE = 16; // how many times per second to execute FSM
  const MINIGAMETV_TIMER_MAX = 20; // seconds
  const MINIGAMETV_WARNING_COLOR1 = '#FF0000';
  const MINIGAMETV_WARNING_COLOR2 = '#FFFF00';
  const MINIGAMETV_MAX_VIBES = 20;

  export class Minigametv extends Minigame {

    private localTimer: Phaser.TimerEvent;
    private localState: number;
    private localCounter: number;
    private localDuration: number;
    private localSeconds: number;
    private localSecondsChangeColor: boolean;

    private backgroundImage: Phaser.Image;
    private balloonSprite: Phaser.Sprite;
    private balloonSprite2: Phaser.Sprite;
    private remoteSprite: Phaser.Sprite;
    private remoteSpriteTween: Phaser.Tween;
    private motherSprite: Phaser.Sprite;
    private channelsSprite: Phaser.Sprite;
    private channelRequestSprite: Phaser.Sprite;
    private timerText: Phaser.Text;
    private vibes: Array<Phaser.Sprite>;
    private requestGroup : Phaser.Group;
    private requestGroup2 : Phaser.Group;

    constructor(level: number){
      super(MINIGAME_TV, level);
    }

    setTargetScore(){
      // this minigame is about choosing one correct image from an image cycle, so 1 point is enough  
      this.targetScore = 1;
    }

    create() {
      this.backgroundImage = this.add.image(this.world.centerX, this.world.centerY, 'minigametv_bg');
      this.backgroundImage.anchor.setTo(0.5,0.5);
      
      this.channelsSprite = this.add.sprite(this.world.centerX+333, this.world.centerY+62, 'channels');
      this.channelsSprite.anchor.set(0.5, 0.5);
      this.channelsSprite.animations.add('idle',[
        0,0,0,0,0,0,0,12,11,10,
        1,1,1,1,1,1,1,12,11,10,
        2,2,2,2,2,2,2,12,11,10,
        3,3,3,3,3,3,3,12,11,10,
        4,4,4,4,4,4,4,12,11,10,
        5,5,5,5,5,5,5,12,11,10,
        6,6,6,6,6,6,6,12,11,13,
        7,7,7,7,7,7,7,12,11,10,
        8,8,8,8,8,8,8,12,11,10,
        9,9,9,9,9,9,9,12,11,10],20,true);
      this.channelsSprite.play('idle');
      
      this.balloonSprite = this.add.sprite(this.world.centerX-110, this.world.centerY-140, 'ui-balloon');
      this.balloonSprite.anchor.set(0.5, 0.5);

      this.channelRequestSprite = this.add.sprite(this.world.centerX-110, this.world.centerY-155, 'channels');
      this.channelRequestSprite.anchor.set(0.5, 0.5);
      this.channelRequestSprite.scale.setTo(0.5, 0.5);
      this.channelRequestSprite.frame = Math.round(Math.random() * 10);
      if (this.channelRequestSprite.frame == 10) {
        this.channelRequestSprite.frame = 9;
      }

      this.requestGroup = this.game.add.group();
      this.requestGroup.add(this.balloonSprite);
      this.requestGroup.add(this.channelRequestSprite);
      this.requestGroup.y = this.world.centerY-220;
      this.game.add.tween(this.requestGroup).to({y: this.world.centerY - 240},1000,Phaser.Easing.Circular.InOut,true,0,-1,true);

      this.motherSprite = this.add.sprite(this.world.centerX-200, this.world.centerY+135, 'mother_sprite');
      this.motherSprite.anchor.setTo(0.5, 0.5);
      this.motherSprite.animations.add('idle', [0,1,2,3,4],8,true);
      this.motherSprite.play('idle');

      this.remoteSprite = this.add.sprite(this.world.centerX+200, this.world.centerY+440, 'remote');
      this.remoteSprite.anchor.set(0.5,0.5);
      this.remoteSprite.animations.add('wait',[0],1,true);
      this.remoteSprite.animations.add('push',[1],1,true);
      this.remoteSprite.play('wait');

      this.balloonSprite2 = this.add.sprite(this.world.centerX-350, this.world.centerY-50, 'ui-balloon');
      this.balloonSprite2.anchor.setTo(0.5, 0.5);
      this.balloonSprite2.scale.x *= -1;

      this.localSeconds = MINIGAMETV_TIMER_MAX;
      this.timerText = this.add.text(this.world.centerX-350,this.world.centerY-60,this.localSeconds.toFixed(2).toString(), timerFont);
      this.timerText.anchor.setTo(0.5,0.5);

      this.requestGroup2 = this.game.add.group();
      this.requestGroup2.add(this.balloonSprite2);
      this.requestGroup2.add(this.timerText);
      this.requestGroup2.y = this.world.centerY-270;
      this.game.add.tween(this.requestGroup2).to({y: this.world.centerY - 290},1000,Phaser.Easing.Circular.InOut,true,500,-1,true);


      this.vibes = new Array<Phaser.Sprite>();
      for (let i=0; i<MINIGAMETV_MAX_VIBES; i++) {
        this.vibes.push(this.add.sprite(this.world.centerX + (Math.random()*1000-500), 
          this.world.centerY + (Math.random()*600-300),
          'bad_vibe'));
        this.vibes[i].anchor.setTo(0.5,0.5);
        this.vibes[i].alpha = 1;
        this.vibes[i].height = 0;
        this.vibes[i].width = 0;
      }

      this.input.mouse.capture = true;

      this.localCounter = 0;
      this.localState = MINIGAMETV_STATE_START;
      this.localTimer = this.game.time.events.loop(Phaser.Timer.SECOND * (1 / MINIGAMETV_FSM_RATE), this.doGameLogic, this);
      this.localTimer.timer.start();   
    }

    doGameLogic() {
      switch(this.localState) {
        case MINIGAMETV_STATE_START:
          this.localState = MINIGAMETV_STATE_INSTRUCTIONS;
          break;
        case MINIGAMETV_STATE_END:
          this.endMiniGame();
          break;
        case MINIGAMETV_STATE_INSTRUCTIONS:
          this.localState = MINIGAMETV_STATE_SHOW_REMOTE_START;
          break;
        case MINIGAMETV_STATE_SHOW_REMOTE_START:
          this.remoteSpriteTween = this.game.add.tween(this.remoteSprite).to( 
              { y: this.world.centerY+200 }, 1000, Phaser.Easing.Circular.Out,true);
          this.remoteSpriteTween.onComplete.add(this.remoteFinishedShowing,this);
          this.localState = MINIGAMETV_STATE_SHOW_REMOTE_WAIT;
          break;
        case MINIGAMETV_STATE_SHOW_REMOTE_WAIT:
          // wait for remote to complete showing...
          break;
        case MINIGAMETV_STATE_SHOW_REMOTE_FINISH:
          // put idle remote animation...
          // prepare game for user interaction
          this.localDuration = MINIGAMETV_TIMER_MAX * MINIGAMETV_FSM_RATE;
          this.localCounter = 0;
          this.timerText.visible = true;
          this.timerText.text = '0.00';
          this.localState = MINIGAMETV_STATE_WAIT_USER_SELECTION;
          break;
        case MINIGAMETV_STATE_WAIT_USER_SELECTION:
          if (this.input.activePointer.leftButton.isDown) {
            if (this.channelsSprite.animations.currentAnim.frame > 9) {
              this.channelsSprite.animations.currentAnim.next(3);
            }
            this.channelsSprite.animations.currentAnim.stop();
            this.localState = MINIGAMETV_STATE_PROCESS_SELECTION;
            this.remoteSprite.play('push');
          }
          // while the user awaits to select the channel requested, change timer
          if (this.localCounter > this.localDuration) {
            this.localState = MINIGAMETV_STATE_FAILURE_START;
          } else {
            this.localCounter++;
          }
          this.localSeconds = MINIGAMETV_TIMER_MAX - (this.localCounter / MINIGAMETV_FSM_RATE);
          if (this.localSeconds < 0) {
            this.localSeconds = 0;
          }
          if (this.localSeconds < 5) {
            if ((this.localCounter % 4) == 0) {
              if (this.localSecondsChangeColor) {
                this.timerText.addColor(MINIGAMETV_WARNING_COLOR1,0);
              } else {
                this.timerText.addColor(MINIGAMETV_WARNING_COLOR2,0);
              }
              this.localSecondsChangeColor = !this.localSecondsChangeColor;
            }
          }
          this.timerText.text = this.localSeconds.toFixed(2).toString();
          break;
        case MINIGAMETV_STATE_PROCESS_SELECTION:
          if (this.channelRequestSprite.frame == this.channelsSprite.frame) {
            this.localState = MINIGAMETV_STATE_SUCCESS_START;
          } else {
            this.localState = MINIGAMETV_STATE_FAILURE_START;
          }
          break;
        case MINIGAMETV_STATE_SUCCESS_START:
          for (let i=0; i<MINIGAMETV_MAX_VIBES; i++) {
            this.vibes[i].loadTexture('good_vibe');
            this.game.add.tween(this.vibes[i]).to(
              { alpha: 0, width: 300, height: 300 },2000, Phaser.Easing.Linear.None,true,0,0);
          }
          this.actualScore = this.targetScore;
          this.localCounter = 0;
          this.localDuration = 2 * MINIGAMETV_FSM_RATE;
          this.localState = MINIGAMETV_STATE_SUCCESS_WAIT;
          break;
        case MINIGAMETV_STATE_SUCCESS_WAIT:
          if (this.localCounter > this.localDuration) {
            this.localState = MINIGAMETV_STATE_END;
          } else {
            this.localCounter++;
          }
          break;
        case MINIGAMETV_STATE_FAILURE_START:
          for (let i=0; i<MINIGAMETV_MAX_VIBES; i++) {
            this.game.add.tween(this.vibes[i]).to(
              { alpha: 0, width: 300, height: 300 },2000, Phaser.Easing.Linear.None,true,0,0);
          }
          this.actualScore = 0;
          this.localCounter = 0;
          this.localDuration = 2 * MINIGAMETV_FSM_RATE;
          this.localState = MINIGAMETV_STATE_FAILURE_WAIT;
          break;
        case MINIGAMETV_STATE_FAILURE_WAIT:
          if (this.localCounter > this.localDuration) {
            this.localState = MINIGAMETV_STATE_END;
          } else {
            this.localCounter++;
          }
          break;
      }
    }

    remoteFinishedShowing() {
      this.localState = MINIGAMETV_STATE_SHOW_REMOTE_FINISH;
    }
  }

}
